"""Custom runner to test overriding runner."""
from django_nose import NoseTestSuiteRunner


class CustomNoseTestSuiteRunner(NoseTestSuiteRunner):
    """Custom test runner, to test overring runner."""



"""
Django settings for testing django-nose.

Configuration is overriden by environment variables:

DATABASE_URL - See https://github.com/kennethreitz/dj-database-url
USE_SOUTH - Set to 1 to include South in INSTALLED_APPS
TEST_RUNNER - Dotted path of test runner to use (can also use --test-runner)
NOSE_PLUGINS - Comma-separated list of plugins to add
"""
from __future__ import print_function
from os import environ, path

import dj_database_url

BASE_DIR = path.dirname(path.dirname(__file__))


def rel_path(*subpaths):
    """Construct the full path given a relative path."""
    return path.join(BASE_DIR, *subpaths)

DATABASES = {
    'default':
        dj_database_url.config(
            default='sqlite:///' + rel_path('testapp.sqlite3'))
}

MIDDLEWARE_CLASSES = ()

INSTALLED_APPS = [
    'django_nose',
    'testapp',
]
if environ.get('USE_SOUTH', '0') in (1, '1'):
    import south  # Raise ImportError if not installed
    assert south
    INSTALLED_APPS.insert(0, 'south')

raw_test_runner = environ.get('TEST_RUNNER')
if raw_test_runner:
    TEST_RUNNER = raw_test_runner
else:
    TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

raw_plugins = environ.get('NOSE_PLUGINS')
if raw_plugins:
    NOSE_PLUGINS = raw_plugins.split(',')

SECRET_KEY = 'ssshhhh'



# coding: utf-8
"""The django_nose module."""
from __future__ import unicode_literals

from django_nose.runner import BasicNoseRunner, NoseTestSuiteRunner
from django_nose.testcases import FastFixtureTestCase
assert BasicNoseRunner
assert FastFixtureTestCase

VERSION = (1, 4, 3)
__version__ = '.'.join(map(str, VERSION))

# Django < 1.2 compatibility.
run_tests = run_gis_tests = NoseTestSuiteRunner



"""Test database access without a database."""
from contextlib import contextmanager
from unittest import TestCase

try:
    from django.db.models.loading import cache as apps
except:
    from django.apps import apps

from nose.plugins.attrib import attr
from django_nose.runner import NoseTestSuiteRunner


class GetModelsForConnectionTests(TestCase):
    """Test runner._get_models_for_connection."""

    tables = ['test_table%d' % i for i in range(5)]

    def _connection_mock(self, tables):
        class FakeIntrospection(object):
            def get_table_list(*args, **kwargs):
                return tables

        class FakeConnection(object):
            introspection = FakeIntrospection()

            def cursor(self):
                return None

        return FakeConnection()

    def _model_mock(self, db_table):
        class FakeModel(object):
            _meta = type('meta', (object,), {'db_table': db_table})()

        return FakeModel()

    @contextmanager
    def _cache_mock(self, tables=[]):
        def get_models(*args, **kwargs):
            return [self._model_mock(t) for t in tables]

        old = apps.get_models
        apps.get_models = get_models
        yield
        apps.get_models = old

    def setUp(self):
        """Initialize the runner."""
        self.runner = NoseTestSuiteRunner()

    def test_no_models(self):
        """For a DB with no tables, return nothing."""
        connection = self._connection_mock([])
        with self._cache_mock(['table1', 'table2']):
            self.assertEqual(
                self.runner._get_models_for_connection(connection), [])

    def test_wrong_models(self):
        """If no tables exists for models, return nothing."""
        connection = self._connection_mock(self.tables)
        with self._cache_mock(['table1', 'table2']):
            self.assertEqual(
                self.runner._get_models_for_connection(connection), [])

    @attr("special")
    def test_some_models(self):
        """If some of the models are in the DB, return matching models."""
        connection = self._connection_mock(self.tables)
        with self._cache_mock(self.tables[1:3]):
            result_tables = [
                m._meta.db_table for m in
                self.runner._get_models_for_connection(connection)]
        self.assertEqual(result_tables, self.tables[1:3])

    def test_all_models(self):
        """If all the models have in the DB, return them all."""
        connection = self._connection_mock(self.tables)
        with self._cache_mock(self.tables):
            result_tables = [
                m._meta.db_table for m in
                self.runner._get_models_for_connection(connection)]
        self.assertEqual(result_tables, self.tables)



# coding: utf-8
"""django-nose utility methods."""
from __future__ import unicode_literals


def process_tests(suite, process):
    """Find and process the suite with setup/teardown methods.

    Given a nested disaster of [Lazy]Suites, traverse to the first level
    that has setup or teardown, and do something to them.

    If we were to traverse all the way to the leaves (the Tests)
    indiscriminately and return them, when the runner later calls them, they'd
    run without reference to the suite that contained them, so they'd miss
    their class-, module-, and package-wide setup and teardown routines.

    The nested suites form basically a double-linked tree, and suites will call
    up to their containing suites to run their setups and teardowns, but it
    would be hubris to assume that something you saw fit to setup or teardown
    at the module level is less costly to repeat than DB fixtures. Also, those
    sorts of setups and teardowns are extremely rare in our code. Thus, we
    limit the granularity of bucketing to the first level that has setups or
    teardowns.

    :arg process: The thing to call once we get to a leaf or a test with setup
        or teardown
    """
    if (not hasattr(suite, '_tests') or
            (hasattr(suite, 'hasFixtures') and suite.hasFixtures())):
        # We hit a Test or something with setup, so do the thing. (Note that
        # "fixtures" here means setup or teardown routines, not Django
        # fixtures.)
        process(suite)
    else:
        for t in suite._tests:
            process_tests(t, process)


def is_subclass_at_all(cls, class_info):
    """Return whether ``cls`` is a subclass of ``class_info``.

    Even if ``cls`` is not a class, don't crash. Return False instead.

    """
    try:
        return issubclass(cls, class_info)
    except TypeError:
        return False


def uses_mysql(connection):
    """Return whether the connection represents a MySQL DB."""
    return 'mysql' in connection.settings_dict['ENGINE']



"""Unload fixtures by truncating tables rather than rebuilding.

A copy of Django 1.3.0's stock loaddata.py, adapted so that, instead of
loading any data, it returns the tables referenced by a set of fixtures so we
can truncate them (and no others) quickly after we're finished with them.
"""

import os
import gzip
import zipfile
from itertools import product

from django.conf import settings
from django.core import serializers
from django.db import router, DEFAULT_DB_ALIAS

try:
    from django.db.models import get_apps
except ImportError:
    from django.apps import apps

    def get_apps():
        """Emulate get_apps in Django 1.9 and later."""
        return [a.models_module for a in apps.get_app_configs()]

try:
    import bz2
    has_bz2 = True
except ImportError:
    has_bz2 = False


def tables_used_by_fixtures(fixture_labels, using=DEFAULT_DB_ALIAS):
    """Get tables used by a fixture.

    Acts like Django's stock loaddata command, but, instead of loading data,
    return an iterable of the names of the tables into which data would be
    loaded.
    """
    # Keep a count of the installed objects and fixtures
    fixture_count = 0
    loaded_object_count = 0
    fixture_object_count = 0
    tables = set()

    class SingleZipReader(zipfile.ZipFile):
        def __init__(self, *args, **kwargs):
            zipfile.ZipFile.__init__(self, *args, **kwargs)
            if settings.DEBUG:
                assert len(self.namelist()) == 1, \
                    "Zip-compressed fixtures must contain only one file."

        def read(self):
            return zipfile.ZipFile.read(self, self.namelist()[0])

    compression_types = {
        None:   open,
        'gz':   gzip.GzipFile,
        'zip':  SingleZipReader
    }
    if has_bz2:
        compression_types['bz2'] = bz2.BZ2File

    app_module_paths = []
    for app in get_apps():
        if hasattr(app, '__path__'):
            # It's a 'models/' subpackage
            for path in app.__path__:
                app_module_paths.append(path)
        else:
            # It's a models.py module
            app_module_paths.append(app.__file__)

    app_fixtures = [
        os.path.join(os.path.dirname(path), 'fixtures')
        for path in app_module_paths]
    for fixture_label in fixture_labels:
        parts = fixture_label.split('.')

        if len(parts) > 1 and parts[-1] in compression_types:
            compression_formats = [parts[-1]]
            parts = parts[:-1]
        else:
            compression_formats = list(compression_types.keys())

        if len(parts) == 1:
            fixture_name = parts[0]
            formats = serializers.get_public_serializer_formats()
        else:
            fixture_name, format = '.'.join(parts[:-1]), parts[-1]
            if format in serializers.get_public_serializer_formats():
                formats = [format]
            else:
                formats = []

        if not formats:
            # stderr.write(style.ERROR("Problem installing fixture '%s': %s is
            # not a known serialization format.\n" % (fixture_name, format)))
            return set()

        if os.path.isabs(fixture_name):
            fixture_dirs = [fixture_name]
        else:
            fixture_dirs = app_fixtures + list(settings.FIXTURE_DIRS) + ['']

        for fixture_dir in fixture_dirs:
            # stdout.write("Checking %s for fixtures...\n" %
            # humanize(fixture_dir))

            label_found = False
            for combo in product([using, None], formats, compression_formats):
                database, format, compression_format = combo
                file_name = '.'.join(
                    p for p in [
                        fixture_name, database, format, compression_format
                    ]
                    if p
                )

                # stdout.write("Trying %s for %s fixture '%s'...\n" % \
                # (humanize(fixture_dir), file_name, fixture_name))
                full_path = os.path.join(fixture_dir, file_name)
                open_method = compression_types[compression_format]
                try:
                    fixture = open_method(full_path, 'r')
                    if label_found:
                        fixture.close()
                        # stderr.write(style.ERROR("Multiple fixtures named
                        # '%s' in %s. Aborting.\n" % (fixture_name,
                        # humanize(fixture_dir))))
                        return set()
                    else:
                        fixture_count += 1
                        objects_in_fixture = 0
                        loaded_objects_in_fixture = 0
                        # stdout.write("Installing %s fixture '%s' from %s.\n"
                        # % (format, fixture_name, humanize(fixture_dir)))
                        try:
                            objects = serializers.deserialize(
                                format, fixture, using=using)
                            for obj in objects:
                                objects_in_fixture += 1
                                cls = obj.object.__class__
                                if router.allow_syncdb(using, cls):
                                    loaded_objects_in_fixture += 1
                                    tables.add(cls._meta.db_table)
                            loaded_object_count += loaded_objects_in_fixture
                            fixture_object_count += objects_in_fixture
                            label_found = True
                        except (SystemExit, KeyboardInterrupt):
                            raise
                        except Exception:
                            fixture.close()
                            # stderr.write( style.ERROR("Problem installing
                            # fixture '%s': %s\n" % (full_path, ''.join(tra
                            # ceback.format_exception(sys.exc_type,
                            # sys.exc_value, sys.exc_traceback)))))
                            return set()
                        fixture.close()

                        # If the fixture we loaded contains 0 objects, assume
                        # that an error was encountered during fixture loading.
                        if objects_in_fixture == 0:
                            # stderr.write( style.ERROR("No fixture data found
                            # for '%s'. (File format may be invalid.)\n" %
                            # (fixture_name)))
                            return set()

                except Exception:
                    # stdout.write("No %s fixture '%s' in %s.\n" % \ (format,
                    # fixture_name, humanize(fixture_dir)))
                    pass

    return tables



# coding: utf-8
# vim: tabstop=4 expandtab autoindent shiftwidth=4 fileencoding=utf-8
"""Provides Nose and Django test case assert functions."""

from __future__ import unicode_literals


def _get_nose_vars():
    """Collect assert_*, ok_, and eq_ from nose.tools."""
    from nose import tools
    new_names = {}
    for t in dir(tools):
        if t.startswith('assert_') or t in ('ok_', 'eq_'):
            new_names[t] = getattr(tools, t)
    return new_names

for _name, _value in _get_nose_vars().items():
    vars()[_name] = _value


def _get_django_vars():
    """Collect assert_* methods from Django's TransactionTestCase."""
    import re
    from django.test.testcases import TransactionTestCase
    camelcase = re.compile('([a-z][A-Z]|[A-Z][a-z])')

    def insert_underscore(m):
            """Insert an appropriate underscore into the name."""
            a, b = m.group(0)
            if b.islower():
                return '_{}{}'.format(a, b)
            else:
                return '{}_{}'.format(a, b)

    def pep8(name):
        """Replace camelcase name with PEP8 equivalent."""
        return str(camelcase.sub(insert_underscore, name).lower())

    class Dummy(TransactionTestCase):
        """A dummy test case for gathering current assertion helpers."""

        def nop():
            """A dummy test to get an initialized test case."""
            pass
    dummy_test = Dummy('nop')

    new_names = {}
    for assert_name in [at for at in dir(dummy_test)
                        if at.startswith('assert') and '_' not in at]:
        pepd = pep8(assert_name)
        new_names[pepd] = getattr(dummy_test, assert_name)
    return new_names


for _name, _value in _get_django_vars().items():
    vars()[_name] = _value


#
# Additional assertions
#

def assert_code(response, status_code, msg_prefix=''):
    """Assert the response was returned with the given status code."""
    if msg_prefix:
        msg_prefix = '%s: ' % msg_prefix

    assert response.status_code == status_code, \
        'Response code was %d (expected %d)' % (
            response.status_code, status_code)


def assert_ok(response, msg_prefix=''):
    """Assert the response was returned with status 200 (OK)."""
    return assert_code(response, 200, msg_prefix=msg_prefix)


def assert_mail_count(count, msg=None):
    """Assert the number of emails sent.

    The message here tends to be long, so allow for replacing the whole
    thing instead of prefixing.
    """
    from django.core import mail

    if msg is None:
        msg = ', '.join([e.subject for e in mail.outbox])
        msg = '%d != %d %s' % (len(mail.outbox), count, msg)
    # assert_equals is dynamicaly added above
    assert_equals(len(mail.outbox), count, msg)  # nopep8



# coding: utf-8
"""Django test runner that invokes nose.

You can use... ::

    NOSE_ARGS = ['list', 'of', 'args']

in settings.py for arguments that you want always passed to nose.

"""
from __future__ import print_function, unicode_literals
import os
import sys
from optparse import make_option, NO_DEFAULT
from types import MethodType

import django
from django.conf import settings
from django.core import exceptions
from django.core.management.base import BaseCommand
from django.core.management.color import no_style
from django.core.management.commands.loaddata import Command
from django.db import connections, transaction, DEFAULT_DB_ALIAS

try:
    from django.db.backends.base.creation import BaseDatabaseCreation
except ImportError:
    # Django < 1.7
    from django.db.backends.creation import BaseDatabaseCreation

try:
    from importlib import import_module
except ImportError:
    # Django < 1.7 and Python < 2.7
    from django.utils.importlib import import_module

try:
    from django.apps import apps
except ImportError:
    # Django < 1.7
    from django.db.models.loading import cache as apps

import nose.core

from django_nose.plugin import DjangoSetUpPlugin, ResultPlugin, TestReorderer
from django_nose.utils import uses_mysql

try:
    any
except NameError:
    def any(iterable):
        for element in iterable:
            if element:
                return True
        return False

try:
    from django.test.runner import DiscoverRunner
except ImportError:
    # Django < 1.8
    from django.test.simple import DjangoTestSuiteRunner as DiscoverRunner


__all__ = ('BasicNoseRunner', 'NoseTestSuiteRunner')


# This is a table of Django's "manage.py test" options which
# correspond to nosetests options with a different name:
OPTION_TRANSLATION = {'--failfast': '-x',
                      '--nose-verbosity': '--verbosity'}


def translate_option(opt):
    if '=' in opt:
        long_opt, value = opt.split('=', 1)
        return '%s=%s' % (translate_option(long_opt), value)
    return OPTION_TRANSLATION.get(opt, opt)


# Django v1.2 does not have a _get_test_db_name() function.
if not hasattr(BaseDatabaseCreation, '_get_test_db_name'):
    def _get_test_db_name(self):
        TEST_DATABASE_PREFIX = 'test_'

        if self.connection.settings_dict['TEST_NAME']:
            return self.connection.settings_dict['TEST_NAME']
        return TEST_DATABASE_PREFIX + self.connection.settings_dict['NAME']

    BaseDatabaseCreation._get_test_db_name = _get_test_db_name


def _get_plugins_from_settings():
    plugins = (list(getattr(settings, 'NOSE_PLUGINS', [])) +
               ['django_nose.plugin.TestReorderer'])
    for plug_path in plugins:
        try:
            dot = plug_path.rindex('.')
        except ValueError:
            raise exceptions.ImproperlyConfigured(
                "%s isn't a Nose plugin module" % plug_path)
        p_mod, p_classname = plug_path[:dot], plug_path[dot + 1:]

        try:
            mod = import_module(p_mod)
        except ImportError as e:
            raise exceptions.ImproperlyConfigured(
                'Error importing Nose plugin module %s: "%s"' % (p_mod, e))

        try:
            p_class = getattr(mod, p_classname)
        except AttributeError:
            raise exceptions.ImproperlyConfigured(
                'Nose plugin module "%s" does not define a "%s"' %
                (p_mod, p_classname))

        yield p_class()


def _get_options():
    """Return all nose options that don't conflict with django options."""
    cfg_files = nose.core.all_config_files()
    manager = nose.core.DefaultPluginManager()
    config = nose.core.Config(env=os.environ, files=cfg_files, plugins=manager)
    config.plugins.addPlugins(list(_get_plugins_from_settings()))
    options = config.getParser()._get_all_options()

    # copy nose's --verbosity option and rename to --nose-verbosity
    verbosity = [o for o in options if o.get_opt_string() == '--verbosity'][0]
    verbosity_attrs = dict((attr, getattr(verbosity, attr))
                           for attr in verbosity.ATTRS
                           if attr not in ('dest', 'metavar'))
    options.append(make_option('--nose-verbosity',
                               dest='nose_verbosity',
                               metavar='NOSE_VERBOSITY',
                               **verbosity_attrs))

    # Django 1.6 introduces a "--pattern" option, which is shortened into "-p"
    # do not allow "-p" to collide with nose's "--plugins" option.
    plugins_option = [
        o for o in options if o.get_opt_string() == '--plugins'][0]
    plugins_option._short_opts.remove('-p')

    django_opts = [opt.dest for opt in BaseCommand.option_list] + ['version']
    return tuple(
        o for o in options if o.dest not in django_opts and o.action != 'help')


if hasattr(BaseCommand, 'use_argparse'):
    # Django 1.8 and later uses argparse.ArgumentParser
    # Translate nose optparse arguments to argparse
    class BaseRunner(DiscoverRunner):

        # Don't pass the following options to nosetests
        django_opts = [
            '--noinput', '--liveserver', '-p', '--pattern', '--testrunner',
            '--settings',
            # 1.8 arguments
            '--keepdb', '--reverse', '--debug-sql',
            # 1.9 arguments
            '--parallel',
        ]

        #
        # For optparse -> argparse conversion
        #
        # Option strings to remove from Django options if found
        _argparse_remove_options = (
            '-p',  # Short arg for nose's --plugins, not Django's --patterns
            '-d',  # Short arg for nose's --detailed-errors, not Django's
                   #  --debug-sql
        )

        # Convert nose optparse options to argparse options
        _argparse_type = {
            'int': int,
            'float': float,
            'complex': complex,
            'string': str,
            'choice': str,
        }
        # If optparse has a None argument, omit from call to add_argument
        _argparse_omit_if_none = (
            'action', 'nargs', 'const', 'default', 'type', 'choices',
            'required', 'help', 'metavar', 'dest')

        # Always ignore these optparse arguments
        # Django will parse without calling the callback
        # nose will then reparse with the callback
        _argparse_callback_options = (
            'callback', 'callback_args', 'callback_kwargs')

        # Keep track of nose options with nargs=1
        _has_nargs = set(['--verbosity'])

        @classmethod
        def add_arguments(cls, parser):
            """Convert nose's optparse arguments to argparse."""
            super(BaseRunner, cls).add_arguments(parser)

            # Read optparse options for nose and plugins
            cfg_files = nose.core.all_config_files()
            manager = nose.core.DefaultPluginManager()
            config = nose.core.Config(
                env=os.environ, files=cfg_files, plugins=manager)
            config.plugins.addPlugins(list(_get_plugins_from_settings()))
            options = config.getParser()._get_all_options()

            # Gather existing option strings`
            django_options = set()
            for action in parser._actions:
                for override in cls._argparse_remove_options:
                    if override in action.option_strings:
                        # Emulate parser.conflict_handler='resolve'
                        parser._handle_conflict_resolve(
                            None, ((override, action),))
                django_options.update(action.option_strings)

            # Process nose optparse options
            for option in options:
                # Skip any options also in Django options
                opt_long = option.get_opt_string()
                if opt_long in django_options:
                    continue
                if option._short_opts:
                    opt_short = option._short_opts[0]
                    if opt_short in django_options:
                        continue
                else:
                    opt_short = None

                # Rename nose's --verbosity to --nose-verbosity
                if opt_long == '--verbosity':
                    opt_long = '--nose-verbosity'

                # Convert optparse attributes to argparse attributes
                option_attrs = {}
                for attr in option.ATTRS:
                    # Ignore callback options
                    if attr in cls._argparse_callback_options:
                        continue

                    value = getattr(option, attr)

                    if attr == 'default' and value == NO_DEFAULT:
                        continue

                    # Rename options for nose's --verbosity
                    if opt_long == '--nose-verbosity':
                        if attr == 'dest':
                            value = 'nose_verbosity'
                        elif attr == 'metavar':
                            value = 'NOSE_VERBOSITY'

                    # Omit arguments that are None, use default
                    if attr in cls._argparse_omit_if_none and value is None:
                        continue

                    # Convert type from optparse string to argparse type
                    if attr == 'type':
                        value = cls._argparse_type[value]

                    # Convert action='callback' to action='store'
                    if attr == 'action' and value == 'callback':
                        action = 'store'

                    # Keep track of nargs=1
                    if attr == 'nargs':
                        assert value == 1, (
                            'argparse option nargs=%s is not supported' %
                            value)
                        cls._has_nargs.add(opt_long)
                        if opt_short:
                            cls._has_nargs.add(opt_short)

                    # Pass converted attribute to optparse option
                    option_attrs[attr] = value

                # Add the optparse argument
                if opt_short:
                    parser.add_argument(opt_short, opt_long, **option_attrs)
                else:
                    parser.add_argument(opt_long, **option_attrs)
else:
    # Django 1.7 and earlier use optparse
    class BaseRunner(DiscoverRunner):
        # Replace the builtin options with the merged django/nose options:
        options = _get_options()

        # Not add following options to nosetests
        django_opts = [
            '--noinput', '--liveserver', '-p', '--pattern', '--testrunner']

        # Default nosetest options with an argument
        _has_nargs = set([
            '--attr', '--config', '--cover-html-dir', '--cover-min-percentage',
            '--cover-package', '--cover-xml-file', '--debug', '--debug-log',
            '--doctest-extension', '--doctest-fixtures', '--doctest-options',
            '--doctest-result-variable', '--eval-attr', '--exclude',
            '--id-file', '--ignore-files', '--include', '--logging-config',
            '--logging-datefmt', '--logging-filter', '--logging-format',
            '--logging-level', '--match', '--process-timeout', '--processes',
            '--profile-restrict', '--profile-sort', '--profile-stats-file',
            '--py3where', '--tests', u'--verbosity', '--where', '--xunit-file',
            '--xunit-testsuite-name', '-A', '-I', '-a', '-c', '-e', '-i', '-l',
            '-m', '-w'])


class BasicNoseRunner(BaseRunner):
    """Facade that implements a nose runner in the guise of a Django runner.

    You shouldn't have to use this directly unless the additions made by
    ``NoseTestSuiteRunner`` really bother you. They shouldn't, because they're
    all off by default.
    """

    __test__ = False

    def run_suite(self, nose_argv):
        """Run the test suite."""
        result_plugin = ResultPlugin()
        plugins_to_add = [DjangoSetUpPlugin(self),
                          result_plugin,
                          TestReorderer()]

        for plugin in _get_plugins_from_settings():
            plugins_to_add.append(plugin)

        try:
            django.setup()
        except AttributeError:
            # Setup isn't necessary in Django < 1.7
            pass

        nose.core.TestProgram(argv=nose_argv, exit=False,
                              addplugins=plugins_to_add)
        return result_plugin.result

    def run_tests(self, test_labels, extra_tests=None):
        """Run the unit tests for all the test names in the provided list.

        Test names specified may be file or module names, and may optionally
        indicate the test case to run by separating the module or file name
        from the test case name with a colon. Filenames may be relative or
        absolute.

        N.B.: The test_labels argument *MUST* be a sequence of
        strings, *NOT* just a string object.  (Or you will be
        specifying tests for for each character in your string, and
        not the whole string.

        Examples:

        runner.run_tests( ('test.module',) )
        runner.run_tests(['another.test:TestCase.test_method'])
        runner.run_tests(['a.test:TestCase'])
        runner.run_tests(['/path/to/test/file.py:test_function'])
        runner.run_tests( ('test.module', 'a.test:TestCase') )

        Note: the extra_tests argument is currently ignored.  You can
        run old non-nose code that uses it without totally breaking,
        but the extra tests will not be run.  Maybe later.

        Returns the number of tests that failed.
        """
        nose_argv = (['nosetests'] + list(test_labels))
        if hasattr(settings, 'NOSE_ARGS'):
            nose_argv.extend(settings.NOSE_ARGS)

        # Skip over 'manage.py test' and any arguments handled by django.
        django_opts = self.django_opts[:]
        for opt in BaseCommand.option_list:
            django_opts.extend(opt._long_opts)
            django_opts.extend(opt._short_opts)

        # Recreate the arguments in a nose-compatible format
        arglist = sys.argv[1:]
        has_nargs = getattr(self, '_has_nargs', set(['--verbosity']))
        while arglist:
            opt = arglist.pop(0)
            if not opt.startswith('-'):
                # Discard test labels
                continue
            if any(opt.startswith(d) for d in django_opts):
                # Discard options handled by Djangp
                continue

            trans_opt = translate_option(opt)
            nose_argv.append(trans_opt)

            if opt in has_nargs:
                # Handle arguments without an equals sign
                opt_value = arglist.pop(0)
                nose_argv.append(opt_value)

        # if --nose-verbosity was omitted, pass Django verbosity to nose
        if ('--verbosity' not in nose_argv and
                not any(opt.startswith('--verbosity=') for opt in nose_argv)):
            nose_argv.append('--verbosity=%s' % str(self.verbosity))

        if self.verbosity >= 1:
            print(' '.join(nose_argv))

        result = self.run_suite(nose_argv)
        # suite_result expects the suite as the first argument.  Fake it.
        return self.suite_result({}, result)


_old_handle = Command.handle


def _foreign_key_ignoring_handle(self, *fixture_labels, **options):
    """Wrap the the stock loaddata to ignore foreign key checks.

    This allows loading circular references from fixtures, and is
    monkeypatched into place in setup_databases().
    """
    using = options.get('database', DEFAULT_DB_ALIAS)
    commit = options.get('commit', True)
    connection = connections[using]

    # MySQL stinks at loading circular references:
    if uses_mysql(connection):
        cursor = connection.cursor()
        cursor.execute('SET foreign_key_checks = 0')

    _old_handle(self, *fixture_labels, **options)

    if uses_mysql(connection):
        cursor = connection.cursor()
        cursor.execute('SET foreign_key_checks = 1')

        if commit:
            connection.close()


def _skip_create_test_db(self, verbosity=1, autoclobber=False, serialize=True,
                         keepdb=True):
    """``create_test_db`` implementation that skips both creation and flushing.

    The idea is to re-use the perfectly good test DB already created by an
    earlier test run, cutting the time spent before any tests run from 5-13s
    (depending on your I/O luck) down to 3.
    """
    # Notice that the DB supports transactions. Originally, this was done in
    # the method this overrides. The confirm method was added in Django v1.3
    # (https://code.djangoproject.com/ticket/12991) but removed in Django v1.5
    # (https://code.djangoproject.com/ticket/17760). In Django v1.5
    # supports_transactions is a cached property evaluated on access.
    if callable(getattr(self.connection.features, 'confirm', None)):
        # Django v1.3-4
        self.connection.features.confirm()
    elif hasattr(self, "_rollback_works"):
        # Django v1.2 and lower
        can_rollback = self._rollback_works()
        self.connection.settings_dict['SUPPORTS_TRANSACTIONS'] = can_rollback

    return self._get_test_db_name()


def _reusing_db():
    """Return whether the ``REUSE_DB`` flag was passed."""
    return os.getenv('REUSE_DB', 'false').lower() in ('true', '1')


def _can_support_reuse_db(connection):
    """Return True if REUSE_DB is a sensible option for the backend."""
    # Perhaps this is a SQLite in-memory DB. Those are created implicitly when
    # you try to connect to them, so our usual test doesn't work.
    return not connection.creation._get_test_db_name() == ':memory:'


def _should_create_database(connection):
    """Return whether we should recreate the given DB.

    This is true if the DB doesn't exist or the REUSE_DB env var isn't truthy.
    """
    # TODO: Notice when the Model classes change and return True. Worst case,
    # we can generate sqlall and hash it, though it's a bit slow (2 secs) and
    # hits the DB for no good reason. Until we find a faster way, I'm inclined
    # to keep making people explicitly saying REUSE_DB if they want to reuse
    # the DB.

    if not _can_support_reuse_db(connection):
        return True

    # Notice whether the DB exists, and create it if it doesn't:
    try:
        # Connections are cached by some backends, if other code has connected
        # to the database previously under a different database name the
        # cached connection will be used and no exception will be raised.
        # Avoiding this by closing connections and setting to null
        for connection in connections.all():
            connection.close()
        connection.connection = None
        connection.cursor()
    except Exception:  # TODO: Be more discerning but still DB agnostic.
        return True
    return not _reusing_db()


def _mysql_reset_sequences(style, connection):
    """Return a SQL statements needed to reset Django tables."""
    tables = connection.introspection.django_table_names(only_existing=True)
    flush_statements = connection.ops.sql_flush(
        style, tables, connection.introspection.sequence_list())

    # connection.ops.sequence_reset_sql() is not implemented for MySQL,
    # and the base class just returns []. TODO: Implement it by pulling
    # the relevant bits out of sql_flush().
    return [s for s in flush_statements if s.startswith('ALTER')]
    # Being overzealous and resetting the sequences on non-empty tables
    # like django_content_type seems to be fine in MySQL: adding a row
    # afterward does find the correct sequence number rather than
    # crashing into an existing row.


class NoseTestSuiteRunner(BasicNoseRunner):
    """A runner that optionally skips DB creation.

    Monkeypatches connection.creation to let you skip creating databases if
    they already exist. Your tests will start up much faster.

    To opt into this behavior, set the environment variable ``REUSE_DB`` to
    "1" or "true" (case insensitive).
    """

    def _get_models_for_connection(self, connection):
        """Return a list of models for a connection."""
        tables = connection.introspection.get_table_list(connection.cursor())
        return [m for m in apps.get_models() if
                m._meta.db_table in tables]

    def setup_databases(self):
        """Setup databases, skipping DB creation if requested and possible."""
        for alias in connections:
            connection = connections[alias]
            creation = connection.creation
            test_db_name = creation._get_test_db_name()

            # Mess with the DB name so other things operate on a test DB
            # rather than the real one. This is done in create_test_db when
            # we don't monkeypatch it away with _skip_create_test_db.
            orig_db_name = connection.settings_dict['NAME']
            connection.settings_dict['NAME'] = test_db_name

            if _should_create_database(connection):
                # We're not using _skip_create_test_db, so put the DB name
                # back:
                connection.settings_dict['NAME'] = orig_db_name

                # Since we replaced the connection with the test DB, closing
                # the connection will avoid pooling issues with SQLAlchemy. The
                # issue is trying to CREATE/DROP the test database using a
                # connection to a DB that was established with that test DB.
                # MySQLdb doesn't allow it, and SQLAlchemy attempts to reuse
                # the existing connection from its pool.
                connection.close()
            else:
                # Reset auto-increment sequences. Apparently, SUMO's tests are
                # horrid and coupled to certain numbers.
                cursor = connection.cursor()
                style = no_style()

                if uses_mysql(connection):
                    reset_statements = _mysql_reset_sequences(
                        style, connection)
                else:
                    reset_statements = connection.ops.sequence_reset_sql(
                        style, self._get_models_for_connection(connection))

                if hasattr(transaction, "atomic"):
                    with transaction.atomic(using=connection.alias):
                        for reset_statement in reset_statements:
                            cursor.execute(reset_statement)
                else:
                    # Django < 1.6
                    for reset_statement in reset_statements:
                        cursor.execute(reset_statement)
                    transaction.commit_unless_managed(using=connection.alias)

                # Each connection has its own creation object, so this affects
                # only a single connection:
                creation.create_test_db = MethodType(
                    _skip_create_test_db, creation)

        Command.handle = _foreign_key_ignoring_handle

        # With our class patch, does nothing but return some connection
        # objects:
        return super(NoseTestSuiteRunner, self).setup_databases()

    def teardown_databases(self, *args, **kwargs):
        """Leave those poor, reusable databases alone if REUSE_DB is true."""
        if not _reusing_db():
            return super(NoseTestSuiteRunner, self).teardown_databases(
                *args, **kwargs)
        # else skip tearing down the DB so we can reuse it next time



# coding: utf-8
"""
Add extra options from the test runner to the ``test`` command.

This enables browsing all the nose options from the command line.
"""
from __future__ import unicode_literals

from django.conf import settings
from django.test.utils import get_runner


if 'south' in settings.INSTALLED_APPS:
    from south.management.commands.test import Command
else:
    from django.core.management.commands.test import Command


# Django < 1.2 compatibility
test_runner = settings.TEST_RUNNER
if test_runner.endswith('run_tests') or test_runner.endswith('run_gis_tests'):
    import warnings
    warnings.warn(
        'Use `django_nose.NoseTestSuiteRunner` instead of `%s`' % test_runner,
        DeprecationWarning)


TestRunner = get_runner(settings)

if hasattr(TestRunner, 'options'):
    extra_options = TestRunner.options
else:
    extra_options = []


class Command(Command):
    """Implement the ``test`` command."""

    option_list = Command.option_list + tuple(extra_options)



# -*- coding: utf-8 -*-
# flake8: noqa
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Question'
        db.create_table('testapp_question', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('question_text', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('pub_date', self.gf('django.db.models.fields.DateTimeField')()),
        ))
        db.send_create_signal('testapp', ['Question'])

        # Adding model 'Choice'
        db.create_table('testapp_choice', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('question', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['testapp.Question'])),
            ('choice_text', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('votes', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal('testapp', ['Choice'])


    def backwards(self, orm):
        # Deleting model 'Question'
        db.delete_table('testapp_question')

        # Deleting model 'Choice'
        db.delete_table('testapp_choice')


    models = {
        'testapp.choice': {
            'Meta': {'object_name': 'Choice'},
            'choice_text': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'question': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['testapp.Question']"}),
            'votes': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'testapp.question': {
            'Meta': {'object_name': 'Question'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'pub_date': ('django.db.models.fields.DateTimeField', [], {}),
            'question_text': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        }
    }

    complete_apps = ['testapp']



"""Django model tests."""

from datetime import datetime

from django.test import TestCase
from testapp.models import Question, Choice


class NoDatabaseTestCase(TestCase):
    """Tests that don't read or write to the database."""

    def test_question_str(self):
        """Test Question.__str__ method."""
        question = Question(question_text="What is your name?")
        self.assertEqual("What is your name?", str(question))

    def test_choice_str(self):
        """Test Choice.__str__ method."""
        choice = Choice(choice_text='My name is Sir Lancelot of Camelot.')
        self.assertEqual('My name is Sir Lancelot of Camelot.', str(choice))


class UsesDatabaseTestCase(TestCase):
    """Tests that read and write to the database."""

    def test_question(self):
        """Test that votes is initialized to 0."""
        question = Question.objects.create(
            question_text="What is your quest?", pub_date=datetime(1975, 4, 9))
        Choice.objects.create(
            question=question, choice_text="To seek the Holy Grail.")
        self.assertTrue(question.choice_set.exists())
        the_choice = question.choice_set.get()
        self.assertEqual(0, the_choice.votes)


class UsesFixtureTestCase(TestCase):
    """Tests that use a test fixture."""

    fixtures = ["testdata.json"]

    def test_fixture_loaded(self):
        """Test that fixture was loaded."""
        question = Question.objects.get()
        self.assertEqual(
            'What is your favorite color?', question.question_text)
        self.assertEqual(datetime(1975, 4, 9), question.pub_date)
        choice = question.choice_set.get()
        self.assertEqual("Blue.", choice.choice_text)
        self.assertEqual(3, choice.votes)



#!/usr/bin/env python
"""Configure enough Django to run the test suite."""
import sys

from django.conf import settings

if not settings.configured:
    settings.configure(
        DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3'}},
        INSTALLED_APPS=['django_nose'],
        MIDDLEWARE_CLASSES=[],
    )


def runtests(*test_labels):
    """Run the selected tests, or all tests if none selected."""
    from django_nose import NoseTestSuiteRunner
    runner = NoseTestSuiteRunner(verbosity=1, interactive=True)
    failures = runner.run_tests(test_labels)
    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])



"""django-nose packaging."""
from __future__ import unicode_literals
import os
from codecs import open
from setuptools import setup, find_packages


def get_long_description(title):
    """Create the long_description from other files."""
    ROOT = os.path.abspath(os.path.dirname(__file__))

    readme = open(os.path.join(ROOT, 'README.rst'), 'r', 'utf8').read()
    body_tag = ".. Omit badges from docs"
    readme_body_start = readme.index(body_tag)
    assert readme_body_start
    readme_body = readme[readme_body_start + len(body_tag):]

    changelog = open(os.path.join(ROOT, 'changelog.rst'), 'r', 'utf8').read()
    old_tag = ".. Omit older changes from package"
    changelog_body_end = changelog.index(old_tag)
    assert changelog_body_end
    changelog_body = changelog[:changelog_body_end]

    bars = '=' * len(title)
    long_description = """
%(bars)s
%(title)s
%(bars)s
%(readme_body)s

%(changelog_body)s

_(Older changes can be found in the full documentation)._
""" % locals()
    return long_description

setup(
    name='django-nose',
    version='1.4.3',
    description='Makes your Django tests simple and snappy',
    long_description=get_long_description('django-nose'),
    author='Jeff Balogh',
    author_email='me@jeffbalogh.org',
    maintainer='Erik Rose',
    maintainer_email='erikrose@grinchcentral.com',
    url='http://github.com/django-nose/django-nose',
    license='BSD',
    packages=find_packages(exclude=['testapp', 'testapp/*']),
    include_package_data=True,
    zip_safe=False,
    install_requires=['nose>=1.2.1'],
    tests_require=['south>=0.7'],
    test_suite='testapp.runtests.runtests',
    # This blows up tox runs that install django-nose into a virtualenv,
    # because it causes Nose to import django_nose.runner before the Django
    # settings are initialized, leading to a mess of errors. There's no reason
    # we need FixtureBundlingPlugin declared as an entrypoint anyway, since you
    # need to be using django-nose to find the it useful, and django-nose knows
    # about it intrinsically.
    # entry_points="""
    #    [nose.plugins.0.10]
    #    fixture_bundler = django_nose.fixture_bundling:FixtureBundlingPlugin
    #    """,
    keywords='django nose django-nose',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Testing'
    ]
)



# coding: utf-8
"""TestCases that enable extra django-nose functionality."""
from __future__ import unicode_literals

from django import test
from django.conf import settings
from django.core import cache, mail
from django.core.management import call_command
from django.db import connections, DEFAULT_DB_ALIAS, transaction

from django_nose.fixture_tables import tables_used_by_fixtures
from django_nose.utils import uses_mysql


__all__ = ('FastFixtureTestCase', )


class FastFixtureTestCase(test.TransactionTestCase):
    """Test case that loads fixtures once rather than once per test.

    Using this can save huge swaths of time while still preserving test
    isolation. Fixture data is loaded at class setup time, and the transaction
    is committed. Commit and rollback methods are then monkeypatched away (like
    in Django's standard TestCase), and each test is run. After each test, the
    monkeypatching is temporarily undone, and a rollback is issued, returning
    the DB content to the pristine fixture state. Finally, upon class teardown,
    the DB is restored to a post-syncdb-like state by deleting the contents of
    any table that had been touched by a fixture (keeping infrastructure tables
    like django_content_type and auth_permission intact).

    Note that this is like Django's TestCase, not its TransactionTestCase, in
    that you cannot do your own commits or rollbacks from within tests.

    For best speed, group tests using the same fixtures into as few classes as
    possible. Better still, don't do that, and instead use the fixture-bundling
    plugin from django-nose, which does it dynamically at test time.
    """

    cleans_up_after_itself = True  # This is the good kind of puppy.

    @classmethod
    def setUpClass(cls):
        """Turn on manual commits. Load and commit the fixtures."""
        if not test.testcases.connections_support_transactions():
            raise NotImplementedError('%s supports only DBs with transaction '
                                      'capabilities.' % cls.__name__)
        for db in cls._databases():
            # These MUST be balanced with one leave_* each:
            transaction.enter_transaction_management(using=db)
            # Don't commit unless we say so:
            transaction.managed(True, using=db)

        cls._fixture_setup()

    @classmethod
    def tearDownClass(cls):
        """Truncate the world, and turn manual commit management back off."""
        cls._fixture_teardown()
        for db in cls._databases():
            # Finish off any transactions that may have happened in
            # tearDownClass in a child method.
            if transaction.is_dirty(using=db):
                transaction.commit(using=db)
            transaction.leave_transaction_management(using=db)

    @classmethod
    def _fixture_setup(cls):
        """Load fixture data, and commit."""
        for db in cls._databases():
            if (hasattr(cls, 'fixtures') and
                    getattr(cls, '_fb_should_setup_fixtures', True)):
                # Iff the fixture-bundling test runner tells us we're the first
                # suite having these fixtures, set them up:
                call_command('loaddata', *cls.fixtures, **{'verbosity': 0,
                                                           'commit': False,
                                                           'database': db})
            # No matter what, to preserve the effect of cursor start-up
            # statements...
            transaction.commit(using=db)

    @classmethod
    def _fixture_teardown(cls):
        """Empty (only) the tables we loaded fixtures into, then commit."""
        if hasattr(cls, 'fixtures') and \
           getattr(cls, '_fb_should_teardown_fixtures', True):
            # If the fixture-bundling test runner advises us that the next test
            # suite is going to reuse these fixtures, don't tear them down.
            for db in cls._databases():
                tables = tables_used_by_fixtures(cls.fixtures, using=db)
                # TODO: Think about respecting _meta.db_tablespace, not just
                # db_table.
                if tables:
                    connection = connections[db]
                    cursor = connection.cursor()

                    # TODO: Rather than assuming that anything added to by a
                    # fixture can be emptied, remove only what the fixture
                    # added. This would probably solve input.mozilla.com's
                    # failures (since worked around) with Site objects; they
                    # were loading additional Sites with a fixture, and then
                    # the Django-provided example.com site was evaporating.
                    if uses_mysql(connection):
                        cursor.execute('SET FOREIGN_KEY_CHECKS=0')
                        for table in tables:
                            # Truncate implicitly commits.
                            cursor.execute('TRUNCATE `%s`' % table)
                        # TODO: necessary?
                        cursor.execute('SET FOREIGN_KEY_CHECKS=1')
                    else:
                        for table in tables:
                            cursor.execute('DELETE FROM %s' % table)

                transaction.commit(using=db)
                # cursor.close()  # Should be unnecessary, since we committed
                # any environment-setup statements that come with opening a new
                # cursor when we committed the fixtures.

    def _pre_setup(self):
        """Disable transaction methods, and clear some globals."""
        # Repeat stuff from TransactionTestCase, because I'm not calling its
        # _pre_setup, because that would load fixtures again.
        cache.cache.clear()
        settings.TEMPLATE_DEBUG = settings.DEBUG = False

        test.testcases.disable_transaction_methods()

        self.client = self.client_class()
        # self._fixture_setup()
        self._urlconf_setup()
        mail.outbox = []

        # Clear site cache in case somebody's mutated Site objects and then
        # cached the mutated stuff:
        from django.contrib.sites.models import Site
        Site.objects.clear_cache()

    def _post_teardown(self):
        """Re-enable transaction methods, and roll back any changes.

        Rollback clears any DB changes made by the test so the original fixture
        data is again visible.

        """
        # Rollback any mutations made by tests:
        test.testcases.restore_transaction_methods()
        for db in self._databases():
            transaction.rollback(using=db)

        self._urlconf_teardown()

        # We do not need to close the connection here to prevent
        # http://code.djangoproject.com/ticket/7572, since we commit, not
        # rollback, the test fixtures and thus any cursor startup statements.

        # Don't call through to superclass, because that would call
        # _fixture_teardown() and close the connection.

    @classmethod
    def _databases(cls):
        if getattr(cls, 'multi_db', False):
            return connections
        else:
            return [DEFAULT_DB_ALIAS]



# coding: utf-8
"""Included django-nose plugins."""
from __future__ import unicode_literals

import sys

from nose.plugins.base import Plugin
from nose.suite import ContextSuite

from django.test.testcases import TransactionTestCase, TestCase

from django_nose.testcases import FastFixtureTestCase
from django_nose.utils import process_tests, is_subclass_at_all


class AlwaysOnPlugin(Plugin):
    """A base plugin that takes no options and is always enabled."""

    def options(self, parser, env):
        """Avoid adding a ``--with`` option for this plugin.

        We don't have any options, and this plugin is always enabled, so we
        don't want to use superclass's ``options()`` method which would add a
        ``--with-*`` option.
        """

    def configure(self, *args, **kw_args):
        """Configure and enable this plugin."""
        super(AlwaysOnPlugin, self).configure(*args, **kw_args)
        self.enabled = True


class ResultPlugin(AlwaysOnPlugin):
    """Captures the TestResult object for later inspection.

    nose doesn't return the full test result object from any of its runner
    methods.  Pass an instance of this plugin to the TestProgram and use
    ``result`` after running the tests to get the TestResult object.
    """

    name = 'result'

    def finalize(self, result):
        """Finalize test run by capturing the result."""
        self.result = result


class DjangoSetUpPlugin(AlwaysOnPlugin):
    """Configures Django to set up and tear down the environment.

    This allows coverage to report on all code imported and used during the
    initialization of the test runner.
    """

    name = 'django setup'
    score = 150

    def __init__(self, runner):
        """Initialize the plugin with the test runner."""
        super(DjangoSetUpPlugin, self).__init__()
        self.runner = runner
        self.sys_stdout = sys.stdout

    def prepareTest(self, test):
        """Create the Django DB and model tables, and do other setup.

        This isn't done in begin() because that's too early--the DB has to be
        set up *after* the tests are imported so the model registry contains
        models defined in tests.py modules. Models are registered at
        declaration time by their metaclass.

        prepareTestRunner() might also have been a sane choice, except that, if
        some plugin returns something from it, none of the other ones get
        called. I'd rather not dink with scores if I don't have to.

        """
        # What is this stdout switcheroo for?
        sys_stdout = sys.stdout
        sys.stdout = self.sys_stdout

        self.runner.setup_test_environment()
        self.old_names = self.runner.setup_databases()

        sys.stdout = sys_stdout

    def finalize(self, result):
        """Finalize test run by cleaning up databases and environment."""
        self.runner.teardown_databases(self.old_names)
        self.runner.teardown_test_environment()


class Bucketer(object):
    """Collect tests into buckets with similar setup requirements."""

    def __init__(self):
        """Initialize the test buckets."""
        # { (frozenset(['users.json']), True):
        #      [ContextSuite(...), ContextSuite(...)] }
        self.buckets = {}

        # All the non-FastFixtureTestCase tests we saw, in the order they came
        # in:
        self.remainder = []

    def add(self, test):
        """Add test into an initialization bucket.

        Tests are bucketed according to its set of fixtures and the
        value of its exempt_from_fixture_bundling attr.
        """
        if is_subclass_at_all(test.context, FastFixtureTestCase):
            # We bucket even FFTCs that don't have any fixtures, but it
            # shouldn't matter.
            key = (frozenset(getattr(test.context, 'fixtures', [])),
                   getattr(test.context,
                           'exempt_from_fixture_bundling',
                           False))
            self.buckets.setdefault(key, []).append(test)
        else:
            self.remainder.append(test)


class TestReorderer(AlwaysOnPlugin):
    """Reorder tests for various reasons."""

    name = 'django-nose-test-reorderer'

    def options(self, parser, env):
        """Add --with-fixture-bundling to options."""
        super(TestReorderer, self).options(parser, env)  # pointless
        parser.add_option('--with-fixture-bundling',
                          action='store_true',
                          dest='with_fixture_bundling',
                          default=env.get('NOSE_WITH_FIXTURE_BUNDLING', False),
                          help='Load a unique set of fixtures only once, even '
                               'across test classes. '
                               '[NOSE_WITH_FIXTURE_BUNDLING]')

    def configure(self, options, conf):
        """Configure plugin, reading the with_fixture_bundling option."""
        super(TestReorderer, self).configure(options, conf)
        self.should_bundle = options.with_fixture_bundling

    def _put_transaction_test_cases_last(self, test):
        """Reorder test suite so TransactionTestCase-based tests come last.

        Django has a weird design decision wherein TransactionTestCase doesn't
        clean up after itself. Instead, it resets the DB to a clean state only
        at the *beginning* of each test:
        https://docs.djangoproject.com/en/dev/topics/testing/?from=olddocs#
        django. test.TransactionTestCase. Thus, Django reorders tests so
        TransactionTestCases all come last. Here we do the same.

        "I think it's historical. We used to have doctests also, adding cleanup
        after each unit test wouldn't necessarily clean up after doctests, so
        you'd have to clean on entry to a test anyway." was once uttered on
        #django-dev.
        """
        def filthiness(test):
            """Return a score of how messy a test leaves the environment.

            Django's TransactionTestCase doesn't clean up the DB on teardown,
            but it's hard to guess whether subclasses (other than TestCase) do.
            We will assume they don't, unless they have a
            ``cleans_up_after_itself`` attr set to True. This is reasonable
            because the odd behavior of TransactionTestCase is documented, so
            subclasses should by default be assumed to preserve it.

            Thus, things will get these comparands (and run in this order):

            * 1: TestCase subclasses. These clean up after themselves.
            * 1: TransactionTestCase subclasses with
                 cleans_up_after_itself=True. These include
                 FastFixtureTestCases. If you're using the
                 FixtureBundlingPlugin, it will pull the FFTCs out, reorder
                 them, and run them first of all.
            * 2: TransactionTestCase subclasses. These leave a mess.
            * 2: Anything else (including doctests, I hope). These don't care
                 about the mess you left, because they don't hit the DB or, if
                 they do, are responsible for ensuring that it's clean (as per
                 https://docs.djangoproject.com/en/dev/topics/testing/?from=
                 olddocs#writing-doctests)

            """
            test_class = test.context
            if (is_subclass_at_all(test_class, TestCase) or
                (is_subclass_at_all(test_class, TransactionTestCase) and
                 getattr(test_class, 'cleans_up_after_itself', False))):
                return 1
            return 2

        flattened = []
        process_tests(test, flattened.append)
        flattened.sort(key=filthiness)
        return ContextSuite(flattened)

    def _bundle_fixtures(self, test):
        """Reorder tests to minimize fixture loading.

        I reorder FastFixtureTestCases so ones using identical sets
        of fixtures run adjacently. I then put attributes on them
        to advise them to not reload the fixtures for each class.

        This takes support.mozilla.com's suite from 123s down to 94s.

        FastFixtureTestCases are the only ones we care about, because
        nobody else, in practice, pays attention to the ``_fb`` advisory
        bits. We return those first, then any remaining tests in the
        order they were received.
        """
        def suite_sorted_by_fixtures(suite):
            """Flatten and sort a tree of Suites by fixture.

            Add ``_fb_should_setup_fixtures`` and
            ``_fb_should_teardown_fixtures`` attrs to each test class to advise
            it whether to set up or tear down (respectively) the fixtures.

            Return a Suite.

            """
            bucketer = Bucketer()
            process_tests(suite, bucketer.add)

            # Lay the bundles of common-fixture-having test classes end to end
            # in a single list so we can make a test suite out of them:
            flattened = []
            for (key, fixture_bundle) in bucketer.buckets.items():
                fixtures, is_exempt = key
                # Advise first and last test classes in each bundle to set up
                # and tear down fixtures and the rest not to:
                if fixtures and not is_exempt:
                    # Ones with fixtures are sure to be classes, which means
                    # they're sure to be ContextSuites with contexts.

                    # First class with this set of fixtures sets up:
                    first = fixture_bundle[0].context
                    first._fb_should_setup_fixtures = True

                    # Set all classes' 1..n should_setup to False:
                    for cls in fixture_bundle[1:]:
                        cls.context._fb_should_setup_fixtures = False

                    # Last class tears down:
                    last = fixture_bundle[-1].context
                    last._fb_should_teardown_fixtures = True

                    # Set all classes' 0..(n-1) should_teardown to False:
                    for cls in fixture_bundle[:-1]:
                        cls.context._fb_should_teardown_fixtures = False

                flattened.extend(fixture_bundle)
            flattened.extend(bucketer.remainder)

            return ContextSuite(flattened)

        return suite_sorted_by_fixtures(test)

    def prepareTest(self, test):
        """Reorder the tests."""
        test = self._put_transaction_test_cases_last(test)
        if self.should_bundle:
            test = self._bundle_fixtures(test)
        return test



"""Models for django-nose test application.

Based on the Django tutorial:
https://docs.djangoproject.com/en/1.8/intro/tutorial01/
"""

from django.db import models


class Question(models.Model):
    """A poll question."""

    question_text = models.CharField(max_length=200)
    pub_date = models.DateTimeField('date published')

    def __str__(self):
        """Return string representation."""
        return self.question_text


class Choice(models.Model):
    """A poll answer."""

    question = models.ForeignKey(Question)
    choice_text = models.CharField(max_length=200)
    votes = models.IntegerField(default=0)

    def __str__(self):
        """Return string representation."""
        return self.choice_text



"""Additional plugins for testing plugins."""
from nose.plugins import Plugin

plugin_began = False


class SanityCheckPlugin(Plugin):
    """Test plugin that registers that it ran."""

    enabled = True

    def options(self, parser, env):
        """Register commandline options."""

    def configure(self, options, conf):
        """Configure plugin."""

    def begin(self):
        """Flag that the plugin was run."""
        global plugin_began
        plugin_began = True



"""Test loading of additional plugins."""
from nose.tools import eq_


def test_one():
    """Test that the test plugin was initialized."""
    from testapp import plugins
    eq_(plugins.plugin_began, True)
